<?php
/**
 * File: $Id: pgsql.php,v 1.6 2005/02/27 14:20:32 greyhead Exp $
 * Ciamos - Content Management System
 * :TODO: add detailed description of this file
 * @package Ciamos
 * @copyright (C) 2004 Ciamos Development Team.
 * @license GPL {@link http://www.gnu.org/licenses/gpl.html}
 * @link http://www.ciamos.com
 * @subpackage Ciamos_xxx
 * @link link to information for the subpackage
 * @author Claudio Poli (kain) (@link http://www.kuht.it)
 */

if ( !defined("SQL_LAYER") ) {
    define("SQL_LAYER", "pgsql");

include_once(SYS_ROOT_PATH."/class/database.php");

//---------------------------------------------------------------------------------------//
/**
* Description
*
* @param type $var description
* @return type description
*/
class Database extends AbsDatabase {

    var $db_connect;
    var $query_log;

//---------------------------------------------------------------------------------------//
/**
* Connects to database
*/
function connect($persistent, $layer, $host, $db_port, $db_name, $user, $pass) {

$connstr = '';

if (!empty($host)) {
    $connstr = 'host=' . $host;
}
if (!empty($db_port)) {
    $connstr .= ' port=' . $db_port;
}
if (isset($db_name)) {
    $connstr .= ' dbname=\'' . addslashes($db_name) . '\'';
}
if (!empty($user)) {
    $connstr .= ' user=\'' . addslashes($user) . '\'';
}
if (!empty($pass)) {
    $connstr .= ' password=\'' . addslashes($pass) . '\'';
}

!empty($persistent) ? $this->db_connect = pg_pconnect($connstr) : $this->db_connect = pg_connect($connstr);
return $this->db_connect;
}

//---------------------------------------------------------------------------------------//
/**
* Close connection
*/
function close() {
    return pg_close($this->db_connect);
}


/**
 * Execute a databse query
 *
 * @param unknown_type $sql
 * @param unknown_type $limit
 * @param unknown_type $start
 * @return unknown
 */
function query($sql, $limit=0, $start=0) 
{
    global $sysConfig;
    
    if ( defined('CIAMOS_DBBACKUP_INCLUDED') && $sysConfig['backups']) {
        $this->backupCheck($sql);
    }
    if ( ($this->debug & 8) || ($this->debug & 16) ) {
        $this->query_log[] = $sql;
    } else {
        $this->query_log[] = 0;
    }

    if ( !empty($limit) ) {
        if (empty($start)) {
            $start = 0;
        }
        $sql = $sql. " OFFSET ".intval($start)." LIMIT ".intval($limit)."";
    }

    if ($this->debug & 16) {
        if ( function_exists('xdebug_call_function') ) {
            if ( function_exists('xdebug_call_class') ) {
                if ($caller_class = xdebug_call_class()) {
                    $output = "<b>".$caller_class." ::</b> ";
                }
            }
            $output .= "<b>".xdebug_call_function()."()</b><br />";
        }
        $output .= count($this->query_log).": ".$sql;

        $this->bg = ($this->bg == 'bg3') ? $this->bg = 'bg1' : $this->bg = 'bg3';
        echo "<div align='left' style='border-top:1px dashed #003030; padding: 3px;' class='".$this->bg."'>".$output."</div>";
    }

    $result = pg_query($this->db_connect, $sql);
    return $result;
}

//---------------------------------------------------------------------------------------//
/**
* Workaround to genId, it takes the next transaction-safe
* Id in a sequence (yep, PostgreSQL doesn't have autoincremental fields)
*/
function genId($sequence) {
    return $this->nextId($sequence);
}

//---------------------------------------------------------------------------------------//
/**
* Raw method to gain next id available, not transaction safe,
* please use genId or nextId in your code.
*/
function getNewId($table, $obj) {

$resource = $this->query("SELECT ".$obj." FROM ".$table." ORDER BY ".$obj." DESC LIMIT 1");

if ($this->num_rows($resource) == 0) {
    $result = 1;
} else {
    list($result) = $this->fetch_row($resource);
    $result = $result+1;
}

return $result;
}

//---------------------------------------------------------------------------------------//
/**
* Returns the affected rows
*/
function affected_rows() {

$result = pg_affected_rows($this->db_connect);
return $result;
}

//---------------------------------------------------------------------------------------//
/**
* Return the number of rows
*/

function num_rows($resource) {

$result = pg_num_rows($resource);
return $result;
}

//---------------------------------------------------------------------------------------//
/**
* Get the name of the specified field in a result
*/

function field_name($resource, $index) {

$result = pg_field_name($resource, $index);
return $result;
}

//---------------------------------------------------------------------------------------//
/**
* Fetch a result row as an associative array, a numeric array, or both
*/

function fetch_array($resource) {

$result = pg_fetch_array($resource);
return $result;
}

//---------------------------------------------------------------------------------------//
/**
* Get a result row as an enumerated array
*/
function fetch_row($resource) {

$result = pg_fetch_row($resource);
return $result;
}

//---------------------------------------------------------------------------------------//
/**
* Fetch a result row as an object
*/
function fetch_object($resource) {

$result = pg_fetch_object($resource);
return $result;
}

//---------------------------------------------------------------------------------------//
/**
* Exists in core, but really not used
*/
function insert_id() {
return 0;
}

//---------------------------------------------------------------------------------------//
/**
* Get the last transaction-safe available Id in a sequence
* If the sequence doesn't exists, we will create it.
*/
function nextId($seq_name, $ondemand = true) {

$seqname = $this->getSequenceName($seq_name);
$repeat = false;
do {
    $sql = "SELECT NEXTVAL('${seqname}')";
    $result = $this->query($sql);
    if ($ondemand && !$result) {
        $repeat = true;
        $result = $this->createSequence($seq_name);
        if (!$result) {
           return "Cannot create sequence";
        }
    } else {
        $repeat = false;
    }
} while ($repeat);
    if (!$result) {
         return "Cannot create sequence";
    }
        $arr = $this->fetch_row($result);
        return $arr[0];
}

//---------------------------------------------------------------------------------------//
/**
* Format the sequence name
*/
function getSequenceName($sqn) {

return sprintf('%s', preg_replace('/[^a-z0-9_]/i', '_', $sqn));
}

//---------------------------------------------------------------------------------------//
/**
* Create a PostgreSQL sequence
*/
function createSequence($seq_name) {

$seqname = $this->getSequenceName($seq_name);
$result = $this->query("CREATE SEQUENCE ${seqname}");
return $result;
}

//---------------------------------------------------------------------------------------//
/**
* Delete a sequence
*/
function dropSequence($sequence) {

$seqname = $this->getSequenceName($sequence);
return $this->query("DROP SEQUENCE ${seqname}");
}

//---------------------------------------------------------------------------------------//
/**
* Get the error number generated from database
*/
function error() {

if ($this->debug & 1) {
    $result = pg_errormessage($this->db_connect);
    return $result;
}
}

//------------------------------------------------------------------------------------------//
} // END CLASS
//------------------------------------------------------------------------------------------//
} // END DEFINE
?>