<?php
/**
 * Ciamos - Content Management System
 * This class is for Ciamos Security
 * @package Ciamos
 * @copyright (C) 2004-2005 Ciamos Development Team.
 * @license GPL {@link http://www.gnu.org/licenses/gpl.html}
 * @link http://www.ciamos.com
 * @subpackage Ciamos_xxx
 * @link link to information for the subpackage
 * @author Thomas Whitecotton (vi0lat0r) (@link http://www.ciamosbase.com)
 *
 */

if ( !defined('CIAMOS_MAINFILE_INCLUDED') ) {
	header("Location: ../../index.php");
	exit();
}

Class Security extends SysHandler 
{
	/**
	 * Function: Security()
	 * ----------------------------------------------------------------
	 * Purpose:			Required function that sets the security log 
	 *						database table
	 * Arguments:			
	 * Returns/Assigns:	$this->table[0] recieves security log table location
	 */ 
	function Security() 
	{
		global $db;

		$this->table[0] = $db->prefix('security_log');
	}

	/*	Function: RedirectERR()
	 *	----------------------------------------------------------------
	 *	Purpose:			Extension function for SysHandler class that 
	 *						sets the redirect error message
	 *	Arguments:			$message
	 *	Returns/Assigns:	Sets $this->error to $message
	 */ 
	function RedirectERR($message='') 
	{
		$this->error = $message;
	}

	/*	Function: RedirectURL()
	 *	----------------------------------------------------------------
	 *	Purpose:			Extension function for SysHandler class that 
	 *						sets the redirect URL location
	 *	Arguments:			$rurl
	 *	Returns/Assigns:	Sets $this->redirect_url to $rurl
	 */ 
	function RedirectURL($rurl) 
	{
		$this->redirect_url = $rurl;
	}

	/*	Function: RedirectTime()
	 *	----------------------------------------------------------------
	 *	Purpose:			Extension function for SysHandler class that 
	 *						sets the redirect time
	 *	Arguments:			$rtime
	 *	Returns/Assigns:	Sets $this->redirect_time to $rtime
	 */ 
	function RedirectTime($rtime) 
	{
		$this->redirect_time = $rtime;
		return $this->redirect_time;
	}

	/*	Function: getRisk()
	*	----------------------------------------------------------------
	*	Purpose:			Extension function for SysHandler class that 
	*						gets the security risk level
	*	Arguments:			$risk
	*	Returns/Assigns:	Sets $this->risk to the $risk
	*						1 = Low
	*						2 = Medium
	*						3 = High
	*/ 
	function getRisk($risk=1) 
	{
		if ( $risk > 3 ) { 
			$risk = 3; 
		} elseif ( !$risk ) {
			$risk = 0;
		} elseif ( $risk < 1 ) {
			$risk = 1;
		}
		$this->risk = $risk;

		return ($this->risk);
	}

	/**
	 *	Function: IP()
	 *	----------------------------------------------------------------
	 *	Purpose:			Extension function for SysHandler class that 
	 *						sets the user's IP
	 *	Arguments:			
	 *	Returns/Assigns:	Sets $this->ip to _REMOTE_ADDR
	 */ 
	function IP() 
	{
		$this->ip = _REMOTE_ADDR;
		return $this->ip;
	}

	/**
	 *	Function: locateIP()
	 *	----------------------------------------------------------------
	 * 	Purpose:			Extension function for SysHandler class that 
	 *						attempts to locate the user's IP
	 *	Arguments:			
	 *	Returns/Assigns:	Returns the uid of the user found, if any
	 */ 
	function locateIP() 
	{
		global $db;

		$etime = strtotime("now -".$this->cmatch_days." days");

		$sql = "
            SELECT uid 
                FROM ".$db->prefix('security_ip_log')." 
                WHERE utime >= ".$etime." AND uip = '".$this->ip."' 
                LIMIT 1";
		$uid = @$db->fetch_row($db->query($sql));
		return $uid[0];
	}

	/*	Function: emailAdmin()
	 *	----------------------------------------------------------------
	 *	Purpose:			Extension function for SysHandler class that 
	 *						emails the site admins to inform them of a new
	 *						security log
	 *	Arguments:			
	 *	Returns/Assigns:	
	 */ 
	function emailAdmin() 
	{
		// for later
	}

	/*	Function: URL()
	*	----------------------------------------------------------------
	*	Purpose:			Extension function for SysHandler class that 
	*						sets the current URL (as seen in the address bar)
	*	Arguments:			
	*	Returns/Assigns:	Sets $this->url to the current loaded URL
	*/ 
	function URL() 
	{
		global $_SERVER;
	
		$this->url = "http://".$_SERVER['SCHEME'].$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'];

		return $this->url;
	}

	/*	Function: RefURL()
	*	----------------------------------------------------------------
	*	Purpose:			Extension function for SysHandler class that 
	*						sets the referring url
	*	Arguments:			
	*	Returns/Assigns:	Sets $this->referrer to the referring URL or
	*						as no referrer information sent.
	*/ 
	function RefURL() 
	{
		$my_url    = 'http://'._HTTP_HOST._PHP_SELF;

		if( (strlen(_HTTP_REFERER) == 0) || (strpos(_HTTP_REFERER, $my_url) != 0) ) {
			return $this->referrer=_NOREFERRER;
		}

		return $this->referrer=_HTTP_REFERER;
	}

	/*	Function: checkRURL()
	*	----------------------------------------------------------------
	*	Purpose:			Extension function for SysHandler class that 
	*						determines if the referring URL is the same as
	*						the installation url.
	*	Arguments:			$rurl
	*	Returns/Assigns:	Sets $this->secure_URL to true if the user was
	*						referred to their current location from a link
	*						on the same website.
	*						Otherwise $this->secure_URL is set to false.
	*/ 
	function checkRURL($rurl='') 
	{
		global $_SERVER;

		$referrer_parts = parse_url($rurl);
		$this->secure_URL = (bool) ($referrer_parts['host'] == $_SERVER['HTTP_HOST']);

		return $this->secure_URL;
	}

	/*	Function: UserID()
	*	----------------------------------------------------------------
	*	Purpose:			Extension function for SysHandler class that 
	*						sets the current user's ID
	*	Arguments:			
	*	Returns/Assigns:	Sets $this->uid to the current user's ID
	*/ 
	function UserID() 
	{
		global $sysUser;
		
		if ( isset($sysUser) ) {
			$this->uid = $sysUser->getVar("uid");
		} else {
			$this->uid = false;
		}

		return $this->uid;
	}

	/*	Function: storeLog()
	*	----------------------------------------------------------------
	*	Purpose:			Extension function for SysHandler class that 
	*						stores the security breach information in the
	*						security log table
	*	Arguments:			
	*	Returns/Assigns:	Inserts the user's ID, IP, current time, security
	*						error, current url location, referring url and
	*						risk level into the security log table
	*/ 
	function storeLog() 
	{
		global $_SERVER, $HTTP_REFERER, $db, $myts, $sysUser, $sysSecurity;

		if ( ( $this->match_ip ) && !$sysUser ) {
			$uid = $this->locateIP();
			unset($this->uid);
			$this->uid = $uid;
		}

		if ( $this->log_low == 0 && $this->risk == 1 ) {
			return (false);
		}

		$sql = "
		    INSERT 
		        INTO ".$this->table[0]." 
			    SET  uid         = '".$this->uid."', 
			         uip         = '$this->ip', 
			         time        = '".date("Y-m-d H:i:s")."', 
			         error       = '".addslashes($this->error)."', 
			         url         = '".$this->url."', 
			         referrer    = '".$this->referrer."',
			         risk        = '".$this->risk."'";
		$db->query($sql);

		if( $this->email_risk >= $this->risk ) {
			$this->emailAdmin();
		}
	}

	/*	Function: SecurityEnabled()
	*	----------------------------------------------------------------
	*	Purpose:			Extension function for SysHandler class that 
	*						checks and sets security preferences
	*	Arguments:			
	*	Returns/Assigns:	Sets $this->security_enabled to $sysSecurity['enabled'];
	*						Sets $this->adv_security_enabled to $sysSecurity['advanced'];
	*						Sets $this->secure_vars to $sysSecurity['secure_vars'];
	*						Sets $this->security_logs to $sysSecurity['log'];
	*						Sets $this->log_low to $sysSecurity['log_low'];
	*						Sets $this->enable_admin to $sysSecurity['enable_admin'];
	*						Sets $this->email_risk to $sysSecurity['email_risk'];
	*						Sets $this->match_ip to $sysSecurity['match_ip'];
	*						Sets $this->cmatch_days to $sysSecurity['cmatch_days'];
	*/ 
	function SecurityEnabled() 
	{
		global $sysSecurity;

		$this->security_enabled       = $sysSecurity['enabled'];
		$this->adv_security_enabled   = $sysSecurity['advanced'];
		$this->security_logs          = $sysSecurity['log'];
		$this->log_low                = $sysSecurity['loglow'];
		$this->enable_admin           = $sysSecurity['enable_admin'];
		$this->email_risk             = $sysSecurity['email_risk'];
		$this->match_ip               = $sysSecurity['match_ip'];
		$this->cmatch_days            = $sysSecurity['cmatch_days'];
	}

	/*	Function: LogLid()
	*	----------------------------------------------------------------
	*	Purpose:			Extension function for SysHandler class that 
	*						sets the log id
	*	Arguments:			
	*	Returns/Assigns:	Sets the $this->log_lid array with all logs lid's
	*/ 
	function LogLid() 
	{
		global $db;

		$sql = "
		    SELECT lid 
		        FROM ".$this->table[0]." 
		        ORDER BY lid ASC";
		$result = $db->query($sql);
		$this->log_id = array();
		$i = 0;
		while ( list($lid) = $db->fetch_row($result) ) {
			$this->log_lid[$i] = $lid;
			$i++;
		}

		return $this->log_lid;
	}

	/*	Function: showSecurityERR()
	*	----------------------------------------------------------------
	*	Purpose:			Extension function for SysHandler class that 
	*						checks if security is enabled and performs 
	*						remedial tasks to determine the information
	*						needed to create a security log.
	*						Also redirects the user to a specified location
	*	Arguments:			$rurl,$rtime,$message,$admin,$redirect,$risk
	*	Returns/Assigns:	If security is enabled, it redirects the user
	*						to a specified url.
	*						If logs are enabled, a log is also created and
	*						entered into the security log database.
	*/ 
	function showSecurityERR($rurl, $rtime, $message='', $admin=0, 
	       $redirect=1, $risk=1) 
	{
		global $_SERVER, $HTTP_REFERER, $db, $myts, $sysUser, $sysHandler;

		$this->SecurityEnabled();
		if ( $this->security_enabled ) {
			$this->RedirectURL($rurl);
			$this->RedirectTime($rtime);
			$this->RedirectERR($message);

			if ( $this->security_logs ) {
				$this->IP();
				$this->URL();
				$this->RefURL();
				$this->UserID();
				$this->getRisk($risk);
				$this->storeLog();
			}
			redirectHeader($this->redirect_url, $this->redirect_time, $this->error, $admin, $redirect);
			exit();
		}
	}

	/*	Function: checkGlobalVars()
	*	----------------------------------------------------------------
	*	Purpose:			Extension function for SysHandler class that 
	*						checks the variables in $sys_request
	*	Arguments:			$exclude,$rurl,$rtime,$message,$index,$risk
	*	Returns/Assigns:	Checks for extra variables that have been
	*						manually typed into the address bar.
	*						If variables are found, the showSecurityERR
	*						function is called, redirecting the user to $rurl
	*/ 
	function checkGlobalVars($rurl=SYS_ROOT_URL, $rtime=2, $message=_NOPERM, $index=0, $risk=0) 
	{
		global $_SERVER, $HTTP_REFERER, $sys_request;

		$this->SecurityEnabled();
		$this->RefURL();

		include_once(SYS_MOD_PATH.'/system/cache/security/allowedvars.php');

		if ( $this->adv_security_enabled && ($this->referrer == _NOREFERRER) && $index ) {
			$showsec = true;
			$risk    = 1;
		} else {
			$stop    = false;
			while( ( list($k,$v) = each($sys_request)) && !$stop ) {
				if ( $allowed != '' ) {
				    $exclusion = explode("|", $allowed);
				}
				if ( isset($k) ) {
				    $security_check = explode($k,_QUERY_STRING);
					if ( !empty($security_check[1]) ) {
						$stop     = true;
						$risk     = 2;
						$tshowsec = $this->redundantChecking($k,$exclusion,$stop);
						if ( !$tshowsec ) {
							$stop = false;
							$risk    = 0;
						}
					}
				}
			}
		}
		if ( $this->adv_security_enabled ) {
			$this->checkRURL($this->referrer);
			if ( $this->secure_URL && $index ) {
				$this->showSecurityERR($rurl,$rtime,$message,0,1,$risk);	
			}
		}
		if ( $showsec ) {
		    $this->showSecurityERR($rurl,$rtime,$message,0,1,$risk);
		}
	}

	/*	Function: redundantChecking()
	*	----------------------------------------------------------------
	*	Purpose:			Extension function for SysHandler class that 
	*						checks the variables in $sys_request
	*	Arguments:			$k,$exclusion,$showsec
	*	Returns/Assigns:	Checks for extra variables that have been
	*						manually typed into the address bar.
	*						If variables are found in the allow list, 0
	*						is returned. If not, the user has been naughty.
	*/ 
	function redundantChecking($k='', $exclusion='', $showsec=0) 
	{
		$i = 0;
		while ( !empty($exclusion[$i]) ){
			if ( $k == $exclusion[$i] ) {
				return(0);
			}
			$i++;
		}
		return $showsec;
	}

	/*	Function: countLogs()
	*	----------------------------------------------------------------
	*	Purpose:			Extension function for SysHandler class that 
	*						counts the total number of logs in the security
	*						log table.
	*	Arguments:			$limit,$where
	*	Returns/Assigns:	Sets $this->total_logs to the total counted
	*						entries in the security log table
	*/ 
	function countLogs($limit=0, $where='') 
	{   
		global $db;

		if ( $limit > 0 ) {
			$limit = " LIMIT ".$limit;
		}
		$sql = "
            SELECT COUNT(*) 
                FROM ".$this->table[0].""
		        .$where
		        .$limit;
		$return = $db->query($sql);
		list($this->total_logs) = $db->fetch_row($return);

		return $this->total_logs;
	}

	/*	Function: showAllLogs()
	*	----------------------------------------------------------------
	*	Purpose:			Extension function for SysHandler class that 
	*						retreives all logs from the security log table
	*	Arguments:			$limit,$where
	*	Returns/Assigns:	Sets $returned to log fields
	*/ 
	function showAllLogs($limit=0, $where='') 
	{
		global $db, $myts, $sysUser;

		if ( $limit == 0 ) {
			$limit = "";
		} else {
			$limit = " LIMIT ".$limit;
		}

		if ( $where == '' ) {
			$where = " WHERE error!=''";
		} else {
			$where = $where." AND error!=''";
		}
		$returned = array();
		$sql = "
		    SELECT * 
		        FROM ".$this->table[0].""
		        .$where." 
		        ORDER BY lid ASC"
		        .$limit;
		$query = $db->query($sql);
		while ( list($return[0], $return[1], $return[2], $return[3], $return[4], 
		    $return[5], $return[6], $return[7]) = $db->fetch_row($query) ) {
			$lid = $return[0];
			for ( $j = 0; $j < 8; $j++ ) {
				$returned[$lid][$j] = $return[$j];
			}
		}
		return $returned;
	}
}
?>