<?php
/**
 * File: $Id: rpc_functions.php,v 1.9 2005/05/12 17:36:10 greyhead Exp $
 *
 *  Ciamos: Simple + Flexible + Advanced Content Management
 *  < http://www.ciamos.com >
 *
 * Global functions used in various parts of the package.
 *
 * @author Half-Dead
 * @copyright 2002 Half-Dead
 * @version 1.2b
 * @license LGPL: (@link /manual/LICENSES/LGPL.txt)
 * @package phpRPC
 */

error_reporting (2039);
set_error_handler('returnError');

/**
 * @access private
 */
function returnError($errno, $errstr, $errfile='', $errline='')
{
    global $phprpc_encoding;

    if ($errno != 8) {

        $serror='<?xml version="1.0" encoding="'.$phprpc_encoding.'"?>' . "\n" .
        '<methodResponse>' . "\n" .
        '  <fault>' . "\n" .
        '    <value>' . "\n" .
        '      <struct>' . "\n" .
        '        <member>' . "\n" .
        '          <name>faultCode</name>' . "\n" .
        '          <value><int>' . $errno . '</int></value>' . "\n" .
        '        </member>' . "\n" .
        '        <member>' . "\n" .
        '          <name>faultString</name>' . "\n" .
        '          <value><string>'.$errstr.' '.$errfile.' '.$errline.'</string></value>' . "\n" .
        '        </member>' . "\n" .
        '      </struct>' . "\n" .
        '    </value>' . "\n" .
        '  </fault>' . "\n" .
        '</methodResponse>';

        header('User-Agent: phpRPC');
        header('Content-Type: text/xml');
        header('Content-length: ' . strlen($serror));
        echo($serror);
        exit();
    }
}

/**
 * Loads extra functions used in various scripts.
 *
 * Usage:
 * i.E: loadFunction("sys.isAdmin");
 *
 * Extra functions are stored in ./include/functions/
 * These can be invoked where needed with this command.
 * Like this extra functions can be added without hacking the core files.
 * @param string $function Loads extra required functions.
 */
function loadFunction($function)
{
    global $phprpc_cms;

    if ( preg_match("/[.]/", $function) ) {
        $path = "./include/functions/$phprpc_cms/" . str_replace('.', '/', $function) . ".php";
    } else {
        $path = "./include/functions/$phprpc_cms/" . $function . ".php";
    }
    if ( @is_file($path) ) {
        require_once($path);
    } else {
        trigger_error(_RC_NOFUNC . $function, E_USER_ERROR);
    }
}

/**
 * Encodes an array to UTF-8.
 * @access private
 * @param $array The array to encode.
 * @return array UTF-8 encoded array
 */
function utf8_encode_array(&$array, $key)
{
    if ( is_array($array) ) {
        array_walk($array, "$this->utf8_encode_array");
    } else {
        $array = utf8_encode($array);
    }
}

/**
 * Decodes a UTF-8 encoded array to ISO-8859-1.
 * @access private
 * @param $array The array to decode.
 * @return array ISO-8859-1 encoded array
 */
function utf8_decode_array(&$array, $key)
{
    if ( is_array($array) ) {
        array_walk($array, "$this->utf8_decode_array");
    } else {
        $array = utf8_decode($array);
    }
}

/**
 * Encodes unix timestamp format to iso8601 time format.
 * @access private
 * @param integer $time Takes unix timestamp as input
 * @return string Returns the datetime in iso8601 format. UTC
 */
function encodeIso8601($time)
{
    $iso8601_Date = gmstrftime("%Y%m%dT%H:%M:%S", $time);
    return $iso8601_Date;
}

/**
 * Decodes iso8601 time format to unix timestamp format
 * @access private
 * @param string $iso8601_Date Takes iso8601 string as input
 * @return integer Returns the datetime in unix timestamp format. UTC
 */
function decodeIso8601($iso8601_Date)
{
    if (ereg("([0-9]{4})([0-9]{2})([0-9]{2})T([0-9]{2}):([0-9]{2}):([0-9]{2})", $iso8601_Date, $regs)) {
        $time = gmmktime($regs[4], $regs[5], $regs[6], $regs[2], $regs[3], $regs[1]);
    } else { $time = time(); }
    return $time;
}

/**
 * Escapes characters above 127 with their numeric html equivalent.
 *
 * @param string $string Numerically escapes string.
 * @return string Escaped string
 */
function escapeNumeric($string)
{
    $trans_array = array();
    for ( $i = 128; $i < 256; $i++ ) {
        $trans_array[chr($i)] = "&#" . $i . ";";
    }
    $string = unhtmlentities($string);
    $string = strtr($string, $trans_array);

    return $string;
}

/**
 * Escapes characters above 127 with their numeric html equivalent.
 *
 * @param string $string .
 * @param string $mode HTML translation table to use: default ENT_QUOTES
 * @return string Numericly escaped string.
 */
function unhtmlentities($string, $mode=ENT_QUOTES)
{
    $trans_tbl = get_html_translation_table (HTML_ENTITIES, $mode);
    $trans_tbl = array_flip ($trans_tbl);

    $string = strtr($string, $trans_tbl);
    $string = preg_replace("/&#([0-9]+);/me", "chr('\\1')", $string);

    return  $string;
}

/**
 * Encodes/Decodes a string
 * ' " < > & are converted to their html counterparts and back.
 *
 * @param string $string String to process.
 * @param string $encode 1/0 Wheteher to encode or decode the string.
 * @return string Encoded or Decoded string.
 */
function xmlQuote($string, $encode = true)
{

    if ( !empty($encode) ) {
        $string = str_replace("'", "&#039;", $string);
        $string = str_replace('"', "&quot;", $string);
        $string = str_replace("<", "&lt;"  , $string);
        $string = str_replace(">", "&gt;"  , $string);
        $string = str_replace("&", "&amp;" , $string);
    } else {
        $string = str_replace("&#039;", "'", $string);
        $string = str_replace("&quot;", '"', $string);
        $string = str_replace("&lt;"  , "<", $string);
        $string = str_replace("&gt;"  , ">", $string);
        $string = str_replace("&amp;" , "&", $string);
    }

    return $string;
}

/**
 * Extracts the text from a given html tag.
 *
 * @param string $tag Tag to look for.
 * @param string $string Input string to process
 * @return string Extracted text
 */
function extractFromTag($tag, $string)
{

    $string = unhtmlentities($string);
    preg_match("/<" . $tag .">(.+?)<\/".$tag.">/is", $string, $match);

    if ( !empty($match[1]) ) {
        return trim($match[1]);
    }
}

/**
 * Dumps data to /cache/logs/ for debugging purposes.
 *
 * @param string $data The output data to write.
 * @param string $name Name of the file to output to.
 */
function dumpData($data, $name)
{
    $logfile = './cache/logs/'.time().'_'.$name;
    $fp = fopen($logfile, 'w');
    fwrite($fp, $data);
    fclose($fp);
}
?>
